/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.util;

//---IMPORTS---
//import uk.org.iscream.cms.util.*;

/**
 * This class monitors a given Queue at regular intervals,
 * reporting back to a Queue with XML - this could be the
 * same Queue being monitored.
 *
 * @author  $Author: tdb $
 * @version $Id: QueueMonitor.java,v 1.11 2003/02/05 14:27:59 tdb Exp $
 */
class QueueMonitor extends Thread {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.11 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---
    
    /**
     * Construct a new QueueMonitor.
     *
     * @param sourceQueue The Queue to monitor
     * @param destQueue The Queue to monitor to
     * @param interval The interval, in milliseconds, at which to sample
     * @param name A name to identify the source Queue with
     */
    public QueueMonitor (Queue sourceQueue, Queue destQueue, long interval, String name) {
        // set the Thread name
        setName("util.QueueMonitor");
        
        _sourceQueue = sourceQueue;
        _destQueue = destQueue;
        _interval = interval;
        _srcName = name;
    }

//---PUBLIC METHODS---
    
    /**
     * Loops continuosly polling our source Queue at
     * the given interval, and then logging the results
     * in the destination Queue.
     */
    public void run() {
        _run = true;
        // so we can stop this thread...
        while(_run) {
            // first sleep for our interval
            try { Thread.sleep(_interval); } catch(Exception e) {}
            // check the Queue
            String status = _sourceQueue.xmlStatus();
            // get a hash of our Queue (for identification)
            String hashCode = String.valueOf(_sourceQueue.hashCode());
            // create some XML
            String date = new Long(System.currentTimeMillis()/((long) 1000)).toString();
            String xml = "<packet type=\"queueStat\" date=\""+date+"\" name=\""+_srcName+"\" hashCode=\""+hashCode+"\">" + status + "</packet>";
            // write XML to destination Queue
            _destQueue.add(xml);
        }
    }
    
    /**
     * Shuts down this QueueMonitor
     */
    public void shutdown() {
        // this will stop the main loop
        _run = false;
        // get a hash of our Queue (for identification)
        String hashCode = String.valueOf(_sourceQueue.hashCode());
        // create some XML
        String date = new Long(System.currentTimeMillis()/((long) 1000)).toString();
        String xml = "<packet type=\"queueStat\" date=\""+date+"\" name=\""+_srcName+"\" hashCode=\""+hashCode+"\" shutdown=\"true\"></packet>";
        // write XML to destination Queue
        _destQueue.add(xml);
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.server.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = null;
    
    /**
     * The Queue we're monitoring.
     */
    private Queue _sourceQueue;
    
    /**
     * The Queue we'll output results to.
     */
    private Queue _destQueue;
    
    /**
     * The interval at which we'll check the
     * sourceQueue status.
     */
    private long _interval;
    
    /**
     * The name to identify the source Queue
     */
    private String _srcName;
    
    /**
     * Allows us to stop the main loop cleanly.
     */
    private boolean _run;
    
//---STATIC ATTRIBUTES---

}
