/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.util;

//---IMPORTS---
import java.io.*;
import org.xml.sax.*;
import javax.xml.parsers.*;

/**
 * XMLPacketMaker - Creates an XMLPacket object.
 *
 * @author  $Author: tdb $
 * @version $Id: XMLPacketMaker.java,v 1.19 2003/02/05 14:27:59 tdb Exp $
 */
public class XMLPacketMaker {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.19 $";

    /** 
     * A reference to the system saxParser factory
     */ 
    private final SAXParserFactory _factory = SAXParserFactory.newInstance();
     
//---STATIC METHODS---

//---CONSTRUCTORS---

//---PUBLIC METHODS---

    /**
     * Method to create an XML packet from the data this
     * class was constructed with.
     *
     * @param xml the XML String to parse
     * @return an XMLPacket representing the XML String given
     * @throws InvalidXMLException if the XML cannot be parsed
     */
    public XMLPacket createXMLPacket(String xml) throws InvalidXMLException {

        // Create the XMLPacket to store values in.
        XMLPacket packet = new XMLPacket();
        
        try {
            // Parse the input
            InputSource inputSource = new InputSource(new StringReader(xml));
            _factory.newSAXParser().parse(inputSource, new XMLStringParser(packet));
        }
        catch (Exception e) {
            // couldn't parse the XML for some reason
            throw new InvalidXMLException("Could not parse the XML: "+xml);
        }
        
        // parsed successfully, return the packet
        return packet;
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.server.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---
    
    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = null;

//---STATIC ATTRIBUTES---

}
