/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.util;

//---IMPORTS---
import java.io.*;
import java.util.LinkedList;
import java.util.Iterator;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import javax.xml.parsers.*;

/**
 * XMLStringParser - Used to assist in creating XMLPacket objects.
 *
 * @author  $Author: tdb $
 * @version $Id: XMLStringParser.java,v 1.17 2003/02/05 14:27:59 tdb Exp $
 */
public class XMLStringParser extends DefaultHandler {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.17 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * No-args constructor.  Generally not used.
     */
    public XMLStringParser () {
        _packet = new XMLPacket();
    }

    /**
     * Constructor for accepting a reference to an XMLPacket
     */
    public XMLStringParser (XMLPacket packet) {
        _packet = packet;
    }

//---PUBLIC METHODS---

    //===========================================================
    // SAX DocumentHandler methods
    //===========================================================

    /** 
     * Add each tag's attribute to the XMLPacket.
     * Note that all attributes within an opening tag are
     * stored as "someroot.sometag.attributes.attribute_name"
     * E.g. If <packet> is the root node, then:
     *     <packet machine_name="raptor">
     * is stored as:
     *     "packet.attributes.machine_name"
     * within the XMLPacket.
     */
    public void startElement (String uri, String name, String qName, Attributes atts) {
        _tagList.addLast(qName);
        if (atts != null) {
            for (int i = 0; i < atts.getLength (); i++) {
                _packet.addParam(getPath()+".attributes."+atts.getQName(i), atts.getValue(i));
            }
        }
    }
    
    /**
     * When an XML element is finished with, we must remove
     * the tag name from the tagList and decrement the indent
     * level.
     */
    public void endElement (String uri, String name, String qName) {
	    _tagList.removeLast();    
    }
    
    /**
     * Any text falling within a pair of terminal tags must
     * be added to the XMLPacket.  Trim leading and trailing
     * spaces and do not bother to add if there is no data
     * specified within the tags.
     */
    public void characters (char[] buf, int offset, int len) throws SAXException {
        String s = new String(buf, offset, len);
        _packet.addParam(getPath(), s);
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.server.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

    //===========================================================
    // Helpers ...
    //===========================================================
    
    /**
     * Return the heirarchical string to be used as a key value
     * in the XMLPacket.
     */
    private String getPath () {
        String path = "";
        if (_tagList.size() > 0) {
            Iterator i = _tagList.iterator();
            path = (String) i.next();
            while(i.hasNext()) {
                path = path + "." + (String) i.next();
            }
        }
        return path;
    }

//---ACCESSOR/MUTATOR METHODS---

    /**
     * Accessor to the XMLPacket.
     */ 
    public XMLPacket getXMLPacket() {
        return _packet;
    }

//---ATTRIBUTES---
    
    /**
     * A LinkedList of tags
     */
    private LinkedList _tagList = new LinkedList();
    
    /**
     * A reference to the XMLPacket we are making
     */
    private XMLPacket _packet;
    
    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = null;

//---STATIC ATTRIBUTES---

}
